@@ -6,6 +6,8 @@ test_description="emacs unthreaded interface"
 
 test_require_emacs
 
+EXPECTED=$NOTMUCH_SRCDIR/test/emacs-unthreaded.expected-output
+
 generate_message "[id]=large-thread-1" '[subject]="large thread"'
 printf "  2001-01-05  Notmuch Test Suite   large thread%43s(inbox unread)\n" >> EXPECTED.unthreaded
 
@@ -34,4 +36,25 @@ output=$(test_emacs '(let ((max-lisp-eval-depth 10))
 		       "SUCCESS")' )
 test_expect_equal "$output" '"SUCCESS"'
 
+add_email_corpus
+test_begin_subtest "Functions in unthreaded-result-format"
+test_emacs '
+(let
+    ((notmuch-unthreaded-result-format
+     (quote (("date" . "%12s  ")
+	     ("authors" . "%-20s")
+	     ("subject" . "%-54s")
+	     (notmuch-test-result-flags . "(%s)")))))
+  (notmuch-unthreaded "tag:inbox")
+  (notmuch-test-wait)
+  (test-output))
+'
+test_expect_equal_file $EXPECTED/result-format-function OUTPUT
+
+test_begin_subtest "notmuch-unthreaded with nonexistent CWD"
+test_emacs '(test-log-error
+	      (let ((default-directory "/nonexistent"))
+	        (notmuch-unthreaded "*")))'
+test_expect_equal "$(cat MESSAGES)" "COMPLETE"
+
 test_done
